/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(2f8896be-232f-4140-afb3-1faffb56f3c6)]
interface nsICacheEntryDoomCallback : nsISupports
{
  /**
   * Callback invoked after an entry or entries has/have been
   * doomed from the cache.
   */
  void onCacheEntryDoomed(in nsresult aResult);
};
